<#
.Description
    Powershell script progettato per caricare una lista di siti intranet o internet e verificare che il certificato SSL installato su ognuno di essi sia in corso di validità. Inoltre, lo script permette di ottenere una tabella ordinata contenente validità e data di scadenza per il certificato di ogni sito internet e, infine, permette di salvare tutto in un file CSV.
.Author 
    Roberto Bonfa'
.Version
    4.3
#>

# Specify the path to your list of URLs
$urlListPath = "./sites.txt"

# Read the URLs from the file
$urls = Get-Content $urlListPath

# Prepare an array to hold the results
$results = @()

# Function to check SSL certificate expiration date
function Get-SslCertificateExpirationDate {
    param([string]$url)

    try {
        $client = New-Object System.Net.Sockets.TcpClient
        $uri = [System.Uri]$url
        $port = $uri.Port -eq -1 ? 443 : $uri.Port
        $client.Connect($uri.Host, $port)

        $sslStream = New-Object System.Net.Security.SslStream($client.GetStream(), $false)
        $sslStream.AuthenticateAsClient($uri.Host)

        $cert = [System.Security.Cryptography.X509Certificates.X509Certificate2]$sslStream.RemoteCertificate
        $expiry = $cert.NotAfter

        $sslStream.Close()
        $client.Close()

        return $expiry
    } catch {
        Write-Host ("Failed to retrieve SSL certificate for {0}: {1}" -f $url, $_.Exception.Message)
        return "Failed to retrieve"
    }
}

foreach ($url in $urls) {
    try {
        # Use Invoke-WebRequest to test the URL
        $response = Invoke-WebRequest -Uri $url -Method Head -TimeoutSec 5 -ErrorAction Stop

        # If no exception is thrown, the site is accessible
        $results += [PSCustomObject]@{
            URL = $url
            Accessible = $true
            StatusDescription = $response.StatusDescription
            StatusCode = $response.StatusCode
            ExpirationDate = "Not Checked" # Placeholder
        }
    } catch {
        # Capture error details
        $results += [PSCustomObject]@{
            URL = $url
            Accessible = $false
            StatusDescription = $_.Exception.Response.StatusDescription
            StatusCode = $_.Exception.Response.StatusCode.value__
            ExpirationDate = "Not Checked" # Placeholder
        }
    }
}

# Optional SSL certificate expiration date check
$userChoiceExpiration = Read-Host "Do you want to verify the expiration of the certificates? (Y/N)"
if ($userChoiceExpiration -eq "Y" -or $userChoiceExpiration -eq "y") {
    foreach ($result in $results) {
        if ($result.Accessible -eq $true) {
            $expirationDate = Get-SslCertificateExpirationDate -url $result.URL
            $result.ExpirationDate = $expirationDate
        } else {
            $result.ExpirationDate = "N/A"
        }
    }
}

# Display results in a table
$results | Format-Table -AutoSize

# Ask if user wants to save results to CSV
$userChoice = Read-Host "Do you want to save these results to a CSV file? (Y/N)"
if ($userChoice -eq "Y" -or $userChoice -eq "y") {
    $csvPath = Read-Host "Enter the full path for the CSV file"
    $results | Export-Csv -Path $csvPath -NoTypeInformation
    Write-Host "Results saved to $csvPath"
} else {
    Write-Host "Results not saved."
}